@@ -25,7 +25,8 @@ class HTTPConflict(HTTPException):
 
 class HTTPForbidden(HTTPException):
     code = 403
-    message = 'A resource with this ID already exists'
+    message = 'You have insufficient permissions to access this resource. If you believe this is invalid, ' \
+              'please contact your system admin.'
 
 
 class HTTPBadRequest(HTTPException):
